@@ -363,7 +363,12 @@ function repo_commit(file)
 end
 
 function url(repo, file; commit=nothing)
-    file = realpath(abspath(file))
+    file = abspath(file)
+    if !isfile(file)
+        @warn "couldn't find file \"$file\" when generating URL"
+        return nothing
+    end
+    file = realpath(file)
     remote = getremote(dirname(file))
     isempty(repo) && (repo = "https://github.com/$remote/blob/{commit}{path}")
     path = relpath_from_repo_root(file)
@@ -658,10 +663,33 @@ function display_dict(x)
     return out
 end
 
+"""
+    struct Default{T}
+
+Internal wrapper type that is meant to be used in situations where it is necessary to
+distinguish whether the user explicitly passed the same value as the default value to a
+keyword argument, or whether the keyword argument was not passed at all.
+
+```julia
+function foo(; kwarg = Default("default value"))
+    if isa(kwarg, Default)
+        # User did not explicitly pass a value for kwarg
+    else kwarg === "default value"
+        # User passed "default value" explicitly
+    end
+end
+```
+"""
+struct Default{T}
+    value :: T
+end
+Base.getindex(default::Default) = default.value
+
 include("DOM.jl")
 include("MDFlatten.jl")
 include("TextDiff.jl")
 include("Selectors.jl")
 include("Markdown2.jl")
+include("JSDependencies.jl")
 
 end
