@@ -84,9 +84,18 @@ const DOCUMENT_STRUCTURE = (
     "subparagraph",
 )
 
+# https://github.com/JuliaLang/julia/pull/32851
+function mktempdir(args...; kwargs...)
+    if VERSION < v"1.3.0-alpha.112"
+        return Base.mktempdir(args...; kwargs...)
+    else
+        return Base.mktempdir(args...; cleanup=false, kwargs...)
+    end
+end
+
 function render(doc::Documents.Document, settings::LaTeX=LaTeX())
     @info "LaTeXWriter: rendering PDF."
-    mktempdir() do path
+    Base.mktempdir() do path
         cp(joinpath(doc.user.root, doc.user.build), joinpath(path, "build"))
         cd(joinpath(path, "build")) do
             name = doc.user.sitename
@@ -125,16 +134,24 @@ function render(doc::Documents.Document, settings::LaTeX=LaTeX())
             end
             cp(STYLE, "documenter.sty")
 
-            # compile .tex and copy over the .pdf file if compile_tex return true
+            # compile .tex
             status = compile_tex(doc, settings, texfile)
-            status && cp(pdffile, joinpath(doc.user.root, doc.user.build, pdffile); force = true)
 
             # Debug: if DOCUMENTER_LATEX_DEBUG environment variable is set, copy the LaTeX
             # source files over to a directory under doc.user.root.
             if haskey(ENV, "DOCUMENTER_LATEX_DEBUG")
-                sources = cp(pwd(), mktempdir(doc.user.root), force=true)
+                dst = isempty(ENV["DOCUMENTER_LATEX_DEBUG"]) ? mktempdir(doc.user.root) :
+                    joinpath(doc.user.root, ENV["DOCUMENTER_LATEX_DEBUG"])
+                sources = cp(pwd(), dst, force=true)
                 @info "LaTeX sources copied for debugging to $(sources)"
             end
+
+            # If the build was successful, copy of the PDF to the .build directory
+            if status
+                cp(pdffile, joinpath(doc.user.root, doc.user.build, pdffile); force = true)
+            else
+                error("Compiling the .tex file failed. See logs for more information.")
+            end
         end
     end
 end
@@ -288,6 +305,10 @@ end
 ## Index, Contents, and Eval Nodes.
 
 function latex(io::IO, index::Documents.IndexNode, page, doc)
+    # Having an empty itemize block in LaTeX throws an error, so we bail early
+    # in that situation:
+    isempty(index.elements) && (_println(io); return)
+
     _println(io, "\\begin{itemize}")
     for (object, _, page, mod, cat) in index.elements
         id = string(hash(string(Utilities.slugify(object))))
@@ -301,22 +322,39 @@ function latex(io::IO, index::Documents.IndexNode, page, doc)
 end
 
 function latex(io::IO, contents::Documents.ContentsNode, page, doc)
+    # Having an empty itemize block in LaTeX throws an error, so we bail early
+    # in that situation:
+    isempty(contents.elements) && (_println(io); return)
+
     depth = 1
-    needs_end = false
     _println(io, "\\begin{itemize}")
     for (count, path, anchor) in contents.elements
         header = anchor.object
         level = Utilities.header_level(header)
         id = string(hash(string(anchor.id, "-", anchor.nth)))
-        level < depth && (_println(io, "\\end{itemize}"); needs_end = false)
-        level > depth && (_println(io, "\\begin{itemize}"); needs_end = true)
+        # If we're changing depth, we need to make sure we always print the
+        # correct number of \begin{itemize} and \end{itemize} statements.
+        if level > depth
+            for k in 1:(level - depth)
+                # if we jump by more than one level deeper we need to put empty
+                # \items in -- otherwise LaTeX will complain
+                (k >= 2) && _println(io, "\\item ~")
+                _println(io, "\\begin{itemize}")
+                depth += 1
+            end
+        elseif level < depth
+            for _ in 1:(depth - level)
+                _println(io, "\\end{itemize}")
+                depth -= 1
+            end
+        end
+        # Print the corresponding \item statement
         _print(io, "\\item \\hyperlink{", id, "}{")
         latexinline(io, header.text)
         _println(io, "}")
-        depth = level
     end
-    needs_end && _println(io, "\\end{itemize}")
-    _println(io, "\\end{itemize}")
+    # print any remaining missing \end{itemize} statements
+    for _ = 1:depth; _println(io, "\\end{itemize}"); end
     _println(io)
 end
 
@@ -336,7 +374,7 @@ function latex(io::IO, d::Dict{MIME,Any})
         _println(io, """
         \\begin{figure}[H]
         \\centering
-        \\includegraphics{$(filename)}
+        \\includegraphics[max width=\\linewidth]{$(filename)}
         \\end{figure}
         """)
     elseif haskey(d, MIME"image/jpeg"())
@@ -344,7 +382,7 @@ function latex(io::IO, d::Dict{MIME,Any})
         _println(io, """
         \\begin{figure}[H]
         \\centering
-        \\includegraphics{$(filename)}
+        \\includegraphics[max width=\\linewidth]{$(filename)}
         \\end{figure}
         """)
     elseif haskey(d, MIME"text/latex"())
@@ -558,7 +596,7 @@ function latexinline(io::IO, md::Markdown.Image)
             normpath(joinpath(dirname(io.filename), md.url))
         end
         url = replace(url, "\\" => "/") # use / on Windows too.
-        wrapinline(io, "includegraphics") do
+        wrapinline(io, "includegraphics[max width=\\linewidth]") do
             _print(io, url)
         end
         _println(io)
