@@ -1,7 +1,10 @@
 #!/bin/sh -e
 
 PKG_JL_BASE=https://api.github.com/repos/JuliaLang/Pkg.jl/tarball/
-PKG_JL_HASH=f71e2c5a119b9c850f9b357fc8c56068f5b51cc0
+PKG_JL_HASH=9419c70af69153cd5c4276bff90fbb135e0aa3f1
+
+STA_JL_BASE=https://api.github.com/repos/JuliaLang/Statistics.jl/tarball/
+STA_JL_HASH=a2203d3b67f7413701be5de251622cb85c9cc69d
 
 LIBUV_BASE=https://api.github.com/repos/JuliaLang/libuv/tarball/
 LIBUV_HASH=35b1504507a7a4168caae3d78db54d1121b121e1
@@ -13,6 +16,10 @@ if ! test -r $PKG_JL_HASH; then
 	wget -c ${PKG_JL_BASE}${PKG_JL_HASH}
 fi
 
+if ! test -r $STA_JL_HASH; then
+	wget -c ${STA_JL_BASE}${STA_JL_HASH}
+fi
+
 if ! test -r libuv-$LIBUV_HASH.tar.gz; then
 	wget -c ${LIBUV_BASE}${LIBUV_HASH} -O libuv-${LIBUV_HASH}.tar.gz
 fi
@@ -32,7 +39,7 @@ foobar () {
 	fi
 }
 
-Documenter=https://github.com/JuliaDocs/Documenter.jl/archive/v0.23.0.tar.gz
+Documenter=https://github.com/JuliaDocs/Documenter.jl/archive/v0.24.4.tar.gz
 foobar Documenter $Documenter
 
 JSON=https://github.com/JuliaIO/JSON.jl/archive/v0.21.0.tar.gz
